from abc import ABC, abstractmethod



class LlmInstance(ABC):
    def __init__(self, name:str):
        self.name = name
        
    
    @abstractmethod
    def generate_response(self, params):
        pass



import hashlib
import mimetypes
import os
import random
import time
import uuid

from agentflow.core.agent import Agent
from agentflow.core.parcel import BinaryParcel
from agents.topics import AgentTopics

import logging
from flowdepot.app_logger import init_logging
logger:logging.Logger = init_logging()

logger.info(f"[FileService] Logger initialized: {logger.name}, Level: {logger.level}")
print(f"[FileService] Logger: {logger.name}, Level: {logger.level}")



class FileService(Agent):
    def __init__(self, name, agent_config):
        logger.info(f"name: {name}, agent_config: {agent_config}")
        super().__init__(name, agent_config)
        self.home_directory = agent_config['home_directory']


    def _generate_file_id(self, filename):        
        current_time = str(int(time.time() * 1000))
        combined_input = filename + current_time + str(random.randint(0, 999)).zfill(3)
        sha1_hash = hashlib.sha1(combined_input.encode()).hexdigest()
        generated_uuid = str(uuid.UUID(sha1_hash[:32])).replace('-', '')
        
        return generated_uuid


    def handle_file_upload(self, topic:str, pcl:BinaryParcel):
        file_info: dict = pcl.content or {}
        content = file_info.get('content')
        logger.info(f"topic: {topic}, filename: {file_info.get('filename')}, content size: {len(content or b'')}")
        # print(f"topic: {topic}, filename: {file_info.get('filename')}, content size: {len(content or b'')}")

        filename = file_info.get('filename')
        if not isinstance(filename, str) or not filename:
            raise ValueError("filename is required and must be a non-empty string")
        file_id = self._generate_file_id(filename)
        mime_type, encoding = mimetypes.guess_type(url=filename)
        logger.debug(f"file_id: {file_id}, filename: {filename}, mime_type: {mime_type}, encoding: {encoding}")
        
        
        file_dir = os.path.join(self.home_directory, file_id[:2], file_id[2:4])
        # file_dir = os.path.join(self.home_directory, file_id[:2])
        if not os.path.exists(file_dir):
            os.makedirs(file_dir)

        file_path = os.path.join(file_dir, f"{file_id}-{filename}")
        content = file_info.get('content')
        open_mode = "w" if isinstance(content, str) else "wb"
        with open(file_path, open_mode) as fp:
            fp.write(content)
        logger.info(f"filename: {filename} is saved.")

        result = {k: v for k, v in file_info.items() if k != 'content'}
        result.update({
            'file_id': file_id,
            'filename': filename,
            'mime_type': mime_type,
            'encoding': encoding,
            'file_path': file_path,
        })
        logger.info(f"result: {result}")
        return result


    def on_activate(self):
        logger.info(f"subscribe: {AgentTopics.FILE_UPLOAD}")
        self.subscribe(AgentTopics.FILE_UPLOAD, "str", self.handle_file_upload)



# -*- coding: utf-8 -*-

import os, sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import time
import unittest
import yaml

import logging
from flowdepot.app_logger import init_logging
logger:logging.Logger = init_logging()

from agentflow.core.agent import Agent
from agentflow.core.parcel import TextParcel, Parcel
from agents.topics import AgentTopics


config_path = os.path.join(os.getcwd(), 'config', 'system.yaml')
with open(config_path, 'r', encoding='utf-8') as f:
    system_config = yaml.safe_load(f) or {}



class TestAgent(unittest.TestCase):
    result_question = ''
    
    class ValidationAgent(Agent):
        def __init__(self):
            super().__init__(name='main', agent_config=system_config)


        def on_activate(self):
            self.subscribe('llm_response')
            
            messages = "Please create a question about the water cycle."
            pcl = TextParcel(messages, 'llm_response')
            self.publish(AgentTopics.LLM_PROMPT , pcl)


        def on_message(self, topic:str, pcl:Parcel):
            llm_resp:dict = pcl.content or {}
            logger.debug(self.M(f"topic: {topic}, llm_resp: {llm_resp}"))

            TestAgent.result_question = llm_resp.get('response', '')


    def setUp(self):
        self.validation_agent = TestAgent.ValidationAgent()
        self.validation_agent.start_thread()


    def _do_test_1(self):
        self.assertTrue(TestAgent.result_question)


    def test_1(self):
        time.sleep(3)

        try:
            self._do_test_1()
        except Exception as ex:
            logger.exception(ex)
            self.assertTrue(False)


    def tearDown(self):
        self.validation_agent.terminate()



if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# @Time    : 2023/10/12 17:00

import os, sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import time
import unittest
import yaml

import logging
from flowdepot.app_logger import init_logging
logger:logging.Logger = init_logging()

from agentflow.core.agent import Agent
from agentflow.core.parcel import BinaryParcel, Parcel
from agents.topics import AgentTopics


config_path = os.path.join(os.getcwd(), 'config', 'system.yaml')
with open(config_path, 'r', encoding='utf-8') as f:
    agent_config = yaml.safe_load(f) or {}



class TestAgent(unittest.TestCase):
    file_id = None
    filename = None
    
    class ValidationAgent(Agent):
        def __init__(self):
            super().__init__(name='main', agent_config=agent_config)


        def on_activate(self):
            self.subscribe('file_uploaded')
            
            filename = 'test_img1.jpg'
            with open(os.path.join(os.getcwd(), 'test', 'data', filename), 'rb') as file:
                content = file.read()
            pcl = BinaryParcel({
                'content': content,
                'filename': filename}, 'file_uploaded')
            self.publish(AgentTopics.FILE_UPLOAD , pcl)


        def on_message(self, topic:str, pcl:Parcel):
            file_info:dict = pcl.content or {}
            logger.debug(self.M(f"topic: {topic}, file_info: {file_info}"))

            TestAgent.file_id = file_info.get('file_id')
            TestAgent.filename = file_info.get('filename')


    def setUp(self):
        home_directory = os.path.join(os.getcwd(), '_upload')
        if not os.path.exists(home_directory):
            os.mkdir(home_directory)

        self.validation_agent = TestAgent.ValidationAgent()
        self.validation_agent.start_thread()


    def _do_test_1(self):
        logger.debug(f'file_id: {TestAgent.file_id}')
        self.assertTrue(TestAgent.file_id)
        self.assertEqual('test_img1.jpg', TestAgent.filename)


    def test_1(self):
        time.sleep(3)

        try:
            self._do_test_1()
        except Exception as ex:
            logger.exception(ex)
            self.assertTrue(False)


    def tearDown(self):
        self.validation_agent.terminate()



if __name__ == '__main__':
    unittest.main()



from agentflow.core.agent import Agent

import logging
from agents import get_logger
logger:logging.Logger = get_logger()



class MokeAgent(Agent):
    def __init__(self, name, agent_config):
        logger.debug(f"name: {name}, agent_config: {agent_config}")
        super().__init__(name, agent_config)



from agents.llm.llms.chatgpt import ChatGPT


def create_instance(name, params):
    if name == ChatGPT.name:
        llm = ChatGPT(params)
    else:
        llm = ChatGPT(params)
        
    return llm



# -*- coding: utf-8 -*-
# @Time    : 2025/06/16 00:56

from datetime import datetime
import logging
import os
import signal
import time
import yaml


LOGGER_NAME = os.environ.get('LOGGER_NAME', 'flowdepot')


def load_config_from_yaml(file_path):
    with open(file_path, 'r', encoding='utf-8') as f:
        return yaml.safe_load(f)

config_path = os.path.join(os.getcwd(), 'config', 'system.yaml')
config = load_config_from_yaml(config_path)


def get_agent_config():
    global config

    broker_name = config['broker']['broker_name']

    agent_config = {
        'version': config['system']['version'],
        'broker': {
            **config['broker'].get(broker_name, {})
        }
    }

    return agent_config


def wait_agent(agent):
    def signal_handler(signal, frame):
        agent.terminate()
    signal.signal(signal.SIGINT, signal_handler)

    time.sleep(1)
    dot_counter = 0
    minute_tracker = datetime.now().minute

    while agent.is_active():
        time.sleep(1)

        dot_counter += 1
        if dot_counter % 6 == 0:
            print('.', end='', flush=True)

        current_minute = datetime.now().minute
        if current_minute != minute_tracker:
            print(f"{datetime.now().strftime('%H:%M')}", end='', flush=True)
            minute_tracker = current_minute
    print()



from openai import OpenAI
import json
import yaml

import logging, os

from agents.llm.llms.base_llm import LlmInstance
logger:logging.Logger = logging.getLogger(os.getenv('LOGGER_NAME'))


class ChatGPT(LlmInstance):
    name = "ChatGPT"
    
    default_params = {
        'model': 'gpt-4o-mini',
        # 'response_format': 'text',  # 'text' or dict (function-calling format)
        'temperature': 0,
        'streaming': False,
        'openai_api_key': "",
    }


    def __init__(self, params: dict):
        super().__init__(name=ChatGPT.name)
        
        self.prompt_params = ChatGPT.default_params.copy()
        self.prompt_params.update(params)

        self.model = self.prompt_params['model']
        self.temperature = self.prompt_params['temperature']
        self.streaming = self.prompt_params['streaming']
        self.api_key = self.prompt_params['openai_api_key']
        self.response_format = self.prompt_params.get('response_format', None)

        self.client = OpenAI(api_key=self.api_key)


    def generate_response(self, prompt_params):
        """
        Generate a response from an OpenAI chat model.

        Args:
            params (str | list[dict] | dict): 
                - If str: treated as a single prompt.
                - If list of dicts: treated as a messages array.
                - If dict: should include a 'messages' key and optionally other settings 
                like 'model', 'temperature', etc.

        Returns:
            str: The generated response text (streamed or full depending on settings).
        """
        if isinstance(prompt_params, str):
            # prompt text only
            messages = [{"role": "user", "content": prompt_params}]
            params = {'messages': messages}
        elif isinstance(prompt_params, list) and isinstance(prompt_params[0], dict):
            # messages list only
            messages = prompt_params
            params = {'messages': messages}
        elif isinstance(prompt_params, dict):
            # must contains 'messages' key
            params = self.prompt_params.copy()
            params.update(prompt_params)
            messages = params['messages']
        else:
            raise ValueError("Invalid input.")

        kwargs = {
            "model": params.get('model', self.model),
            "messages": messages,
            "temperature": params.get('temperature', self.temperature),
            "stream": params.get('streaming', self.streaming),
        }
        response_format = params.get('response_format', self.response_format)
        if response_format:
            kwargs['response_format'] = response_format

        response = self.client.chat.completions.create(**kwargs)

        if self.streaming:
            result = ""
            for chunk in response:
                if chunk.choices and chunk.choices[0].delta.content:
                    result += chunk.choices[0].delta.content
            return result
        else:
            choice = response.choices[0]
            return choice.message.content


if __name__ == '__main__':
    prompt_text = "Please create a question about the water cycle."
    
    messages=[
        {"role": "system", "content": "You are a helpful exam question generator. Please provide your response in JSON format."},
        {"role": "user", "content": f"{prompt_text}\nPlease provide your response in JSON format."}
    ]
    
    response_format={
        "type": "json_schema",
        "json_schema": {
            "name": "generate_question",
            "schema": {
                "type": "object",
                "properties": {
                    "stem": {"type": "string"},
                    "option_A": {"type": "string"},
                    "option_B": {"type": "string"},
                    "option_C": {"type": "string"},
                    "option_D": {"type": "string"},
                    "answer": {"type": "string"},
                },
                "required": [
                    "stem",
                    "option_A",
                    "option_B",
                    "option_C",
                    "option_D",
                    "answer"
                ],
                "additionalProperties": False
            }
        }
    }
    
    config_path = os.path.join(os.getcwd(), 'config', 'system.yaml')
    with open(config_path, 'r', encoding='utf-8') as f:
        params = (yaml.safe_load(f) or {}).get('llm', {})

    llm = ChatGPT(params)
    messages = "Please create a question about the water cycle."
    result = llm.generate_response(messages)
    # params['messages'] = messages
    # result = llm.generate_response(params)
    print(json.dumps(result, indent=2) if isinstance(result, dict) else result)



import re
from urllib import response
import magic
import os
from pathlib import Path
import tempfile
import torch
import whisper

from agentflow.core.agent import Agent
# from agentflow.core.parcel import TextParcel
from agentflow.core.parcel import BinaryParcel
from agents.topics import AgentTopics

import logging
from flowdepot.app_logger import init_logging
logger:logging.Logger = init_logging()



class SttService(Agent):
    def __init__(self, name, agent_config):
        logger.info(f"name: {name}, agent_config: {agent_config}")
        super().__init__(name, agent_config)
        self.whisper_model_name = agent_config["whisper_model"]
        
        # Create "temp" folder in current execution path if it doesn't exist for audio files.
        self.temp_root = Path.cwd() / "temp"
        self.temp_root.mkdir(exist_ok=True)


    def on_activate(self):
        device = "cuda" if torch.cuda.is_available() else "cpu"
        logger.warning(f'Device of Whisper: {device}')

        # Continue here..
        logger.warning(f'Loading model: {self.whisper_model_name}')
        self.whisper_model = whisper.load_model(self.whisper_model_name, device=device)

        self.subscribe(AgentTopics.STT_CONTENT, "str", self.transcribe_content)


    def transcribe_content(self, topic:str, pcl:BinaryParcel):
        audio_info: dict = pcl.content or {}
        content = audio_info.get('content')

        mime = magic.Magic(mime=True)
        response = {}
        try:
            file_mime_type = mime.from_buffer(content)
            logger.info(f'file_mime_type: {file_mime_type}')
            if file_mime_type.startswith('audio/') or file_mime_type.startswith('video/'):
                response['text'] = self._transcribe_content(topic, content, file_mime_type.split('/')[-1])
                response['mime_type'] = file_mime_type
                response['topic'] = topic
            else:
                logger.warning(f'Content is not audio or video.')
        except Exception as ex:
            logger.exception(ex)
            response['error'] = str(ex)
            
        return response
                        
        


    def _transcribe_content(self, _, content, audio_type):
        with tempfile.NamedTemporaryFile(mode="wb", suffix=f".{audio_type}", delete=False) as tmp:
            tmp.write(content)
            tmp.flush()
            file_path = Path(tmp.name)

        try:
            result = self.whisper_model.transcribe(str(file_path))
            transcribed_text = result["text"]
        finally:
            # 確保即使轉換錯誤也會清理檔案
            if file_path.exists():
                os.remove(file_path)

        return transcribed_text
    
    
def main():
    # 初始化 STT Agent
    agent_config = {
        "whisper_model": "base"  # tiny, base, small, medium, large
    }
    stt_agent = SttService("stt-agent", agent_config)
    stt_agent.on_activate()

    # 載入測試音檔
    audio_path = Path("agents/stt/sample_apeech.mp3")
    with audio_path.open("rb") as f:
        content = f.read()

    # 模擬接收到的 TextParcel
    parcel = BinaryParcel(content=content)

    # 呼叫轉錄函式
    topic = "Test/STT"
    result = stt_agent.transcribe_content(topic, parcel)

    print("📜 Transcribed text:")
    print(result)


if __name__ == "__main__":
    main()
    


from agentflow.core.agent import Agent
from agentflow.core.parcel import TextParcel
from agents.llm.llms import create_instance as create_llm
from agents.llm.llms.base_llm import LlmInstance
from agents.topics import AgentTopics

import logging
from flowdepot.app_logger import init_logging
logger:logging.Logger = init_logging()



class LlmService(Agent):
    def __init__(self, name, agent_config):
        logger.info(f"name: {name}, agent_config: {agent_config}")
        super().__init__(name, agent_config)
        self.llm_params = agent_config


    def on_activate(self):
        self.llm:LlmInstance = create_llm(self.llm_params['llm'], self.llm_params)
        self.subscribe(AgentTopics.LLM_PROMPT, "str", self.handle_prompt)


    def handle_prompt(self, topic:str, pcl:TextParcel):
        params = pcl.content

        response = self.llm.generate_response(params)
        logger.debug(self.M(response))

        return {
            'response': response,
        }



from setuptools import setup, find_packages

setup(
    name="flowdepot",
    version="2025.6.8.2343",
    packages=find_packages(),  # 掃描根目錄
    include_package_data=True,
    install_requires=[
        # 在這裡加入依賴，例如：
        # "paho-mqtt>=1.6.1",
        # "pyyaml>=6.0",
    ],
    entry_points={
        # 如需指令列工具：
        # 'console_scripts': [
        #     'flowdepot=agents.main:main',
        # ],
    },
    python_requires='>=3.11',
)
